{
*****************************************************************************
*  Name    : USB StopWatch                                                  *
*  Author  : Hiroyuki Iizuka                                                *
*  Date    : 2014/07/29                                                     *
*  Version : 0.91                                                           *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Program USBStopWatch2

'---- configuration ---------------------------------------------------------
Device = 18F4550
Clock  = 48

Config
   USBDIV  = 1, 
   PLLDIV  = 5,
   CPUDIV  = OSC1_PLL2,
   FOSC    = HSPLL_HS,
   MCLRE   = OFF

'---- include file ----------------------------------------------------------
Include "Convert.bas"
Include "EEPROM.bas"

'---- interrupt -------------------------------------------------------------
#option org_reset     = $2000
#option vector_isr_hi = $2008
#option vector_isr_lo = $2018

Const ipLow  = 1
Const ipHigh = 2

Const High_priority = 1
Const Low_priority  = 0

Dim Interrupt_HIGH_enable As INTCON.booleans(7)
Dim Interrupt_LOW_enable  As INTCON.booleans(6)
Dim Interrupt_priority    As RCON.booleans(7)

'------------ Ir Sensor -----------------------------------------------------
Dim Ir1       As PORTB.0
Dim Ir2       As PORTB.1

'------------ ROTARY ENCODER ------------------------------------------------
Dim ENCODER_A As PORTA.0
Dim ENCODER_B As PORTA.1

Const CW             =  1
Const CCW            = -1
Const ENCODER_STOP   =  0

Const ROTARY_LOCK    = 1
Const ROTARY_UNLOCK  = 0

Dim Rotary_Enc_A   As Byte
Dim Rotary_Enc_B   As Byte
Dim Rotary_A_Lock  As Boolean
Dim Rotary_B_Lock  As Boolean
Dim Rotary_Stat    As ShortInt
Dim turn           As ShortInt

'------------ Push Switch ---------------------------------------------------
Dim SW_START As PORTB.4  ' SW1
Dim SW1      As PORTB.4  ' SW1
Dim SW_SPLIT As PORTA.3  ' SW2 
Dim SW2      As PORTA.3  ' SW2 

Const SW_ON      = 0
Const SW_OFF     = 1

Const SW_ENABLE  = 0
Const SW_DISABLE = 1

'------------ Mechanical RELAY ---------------------------------------------
Dim RELAY As PORTA.5

Const RELAY_ON  = 0
Const RELAY_OFF = 1

'------------ 3mm LED ------------------------------------------------------
Dim LED1 As PORTB.5
Dim LED2 As PORTB.3
Dim LED3 As PORTA.2


'---- 7segment LED ---------------------------------------------------------
Dim LED_DEV6     As PORTC.0
Dim LED_DEV5     As PORTC.1
Dim LED_DEV4     As PORTC.2
Dim LED_DEV3     As PORTE.0
Dim LED_DEV2     As PORTE.1
Dim LED_DEV1     As PORTE.2

Dim LED_SEG_PORT As PORTD
Dim LED_SEG_A    As PORTD.0
Dim LED_SEG_B    As PORTD.1
Dim LED_SEG_C    As PORTD.2
Dim LED_SEG_D    As PORTD.3
Dim LED_SEG_E    As PORTD.4
Dim LED_SEG_F    As PORTD.5
Dim LED_SEG_G    As PORTD.6
Dim LED_SEG_DP   As PORTD.7

Const Digit_N     = $00   ' Blank
Const Digit_0     = $3F
Const Digit_1     = $06
Const Digit_2     = $5B
Const Digit_3     = $4F
Const Digit_4     = $66
Const Digit_5     = $6D
Const Digit_6     = $7D
Const Digit_7     = $07
Const Digit_8     = $7F
Const Digit_9     = $6F
Const Digit_A     = $77
Const Digit_G     = $3D
Const Digit_r     = $50
Const Digit_o     = $5C
Const Digit_P     = $73
Const Digit_S     = $6D
Const Digit_u     = $1C
Const Digit_F     = $71
Const Digit_L     = $38
Const Digit_C     = $39
Const Digit_d     = $5E
Const Digit_E     = $79
Const Digit_H     = $76
Const Digit_MINUS = $40
Const Digit_DP    = $80

Const LED_ON        = 1
Const LED_OFF       = 0

Dim led_val1 As Byte
Dim led_val2 As Byte
Dim led_val3 As Byte
Dim led_val4 As Byte
Dim led_val5 As Byte
Dim led_val6 As Byte
Dim led_pos  As Byte

'-------- TMR0 ------------------------------------------
Dim Timer0_overflowd             As INTCON.booleans(2)
Dim Timer0_interrupt_enable      As INTCON.booleans(5)
Dim Timer0_priority              As INTCON2.bits(2)
Dim Timer0_enable                As T0CON.booleans(7)
Dim Timer0_mode_8bit             As T0CON.booleans(6)
Dim Timer0_use_external_clock    As T0CON.booleans(5)
Dim Timer0_prescale_NOT_assign   As T0CON.booleans(3)
Dim Timer0_prescale_bit2         As T0CON.bits(2)
Dim Timer0_prescale_bit1         As T0CON.bits(1)
Dim Timer0_prescale_bit0         As T0CON.bits(0)

{
'-------- TMR1 ------------------------------------------
Dim Timer1_overflowd             As PIR1.booleans(0) 
Dim Timer1_interrupt_enable      As PIE1.booleans(0) 
Dim Timer1_prescale_bit0         As T1CON.bits(4)
Dim Timer1_prescale_bit1         As T1CON.bits(5)
Dim Timer1_use_external_clock    As T1CON.booleans(1)
Dim Timer1_enable                As T1CON.booleans(0)
Dim Timer1_priority              As IPR1.bits(0)
}

'-------- TMR2 ------------------------------------------
Dim Timer2_overflowd             As PIR1.booleans(1) 
Dim Timer2_interrupt_enable      As PIE1.booleans(1) 
Dim Timer2_prescale_bit0         As T2CON.bits(0)
Dim Timer2_prescale_bit1         As T2CON.bits(1)
Dim Timer2_enable                As T2CON.booleans(2)
Dim Timer2_postscale_bit0        As T2CON.bits(3)
Dim Timer2_postscale_bit1        As T2CON.bits(4)
Dim Timer2_postscale_bit2        As T2CON.bits(5)
Dim Timer2_postscale_bit3        As T2CON.bits(6)
Dim Timer2_priority              As IPR1.bits(1)

'-------- A/D converter ---------------------------------
Dim ADconfig3 As ADCON1.3
Dim ADconfig2 As ADCON1.2
Dim ADconfig1 As ADCON1.1
Dim ADconfig0 As ADCON1.0

'-------- comparator ------------------------------------
Dim Comparator_mode2 As CMCON.bits(2)
Dim Comparator_mode1 As CMCON.bits(1)
Dim Comparator_mode0 As CMCON.bits(0)

'-------- PORTB -----------------------------------------
Dim PortB_pullups_disable As INTCON2.booleans(7)

'---- stop watch ----------------------------------------
Dim time_count     As LongWord

Const MAX_TIME = 599999

Const MESSAGE_TIME_ATTACK_GROUP       = 1
Const MESSAGE_LAPTIME_SINGLE          = 2
Const MESSAGE_BAR                     = 3
  
Dim stopwatch_mode As Byte
Const MODE_TIME_ATTACK_GROUP  = 0
Const MODE_LAPTIME_SINGLE     = 1

Dim race_number_start  As Byte
Dim race_number_goal   As Byte
Dim raceing_number     As ShortInt
Dim start_time(10)     As LongWord
Dim goal_time(10)      As LongWord
Dim temp_start_time    As LongWord
Dim temp_goal_time     As LongWord
Dim start_time_prev    As LongWord
Dim goal_time_prev     As LongWord

Dim dead_time            As LongWord
Dim display_time         As LongWord
Dim display_down_counter As LongWord

Dim i As Byte

'---- EEPROM initialize -----------------------------------------------------
Const DEFAULT_DEAD_TIME    = 100 * 5   // 1[sec]
Const DEFAULT_DISPLAY_TIME = 300 * 5   // 3[sec]
EEPROM = (MODE_TIME_ATTACK_GROUP As Byte, DEFAULT_DEAD_TIME As LongWord, DEFAULT_DISPLAY_TIME As LongWord )

Const eepaddr_stopwatch_mode = 0
Const eepaddr_dead_time      = 1
Const eepaddr_display_time   = 5


'===============================================================================
' interrupt priority HIGH     Overflow 2 ms
'===============================================================================
Interrupt OnTimerHigh(ipHigh)
  '---- TMR2 overflow ----------
  If Timer2_overflowd Then
    time_count = time_count + 1
    Timer2_overflowd = false
    If display_down_counter > 0 Then
      display_down_counter = display_down_counter - 1
    End If
  End If
End Interrupt


'===============================================================================
' interrupt priority LOW
'===============================================================================
Interrupt OnTimerLow(ipLow)
  '--- TMR0 overflow (7SEG_LED storobe)   2 msec
  If Timer0_overflowd Then
    TMR0L = 69
    Select led_pos
      Case 1
        LED_DEV6 = LED_OFF
        LED_SEG_PORT = led_val1
        LED_DEV1 = LED_ON
        led_pos = led_pos + 1
      Case 2
        LED_DEV1 = LED_OFF
        LED_SEG_PORT = led_val2
        LED_DEV2 = LED_ON
        led_pos = led_pos + 1
      Case 3
        LED_DEV2 = LED_OFF
        LED_SEG_PORT = led_val3
        LED_DEV3 = LED_ON
        led_pos = led_pos + 1
      Case 4
        LED_DEV3 = LED_OFF
        LED_SEG_PORT = led_val4
        LED_DEV4 = LED_ON
        led_pos = led_pos + 1
      Case 5
        LED_DEV4 = LED_OFF
        LED_SEG_PORT = led_val5
        LED_DEV5 = LED_ON
        led_pos = led_pos + 1
      Case 6
        LED_DEV5 = LED_OFF
        LED_SEG_PORT = led_val6
        LED_DEV6 = LED_ON
        led_pos = 1
    EndSelect
    Timer0_overflowd = false   ' Clear interrupt flag
  End If
End Interrupt

'============================================================================
' Rotary encoder
'
' RETURN
'    CW / CCW / OTHER
'============================================================================
Function rotary_encoder() As ShortInt
  Dim temp_A As Byte
  Dim temp_B As Byte

  temp_A = ENCODER_A
  temp_B = ENCODER_B

  result = ENCODER_STOP

  If (Rotary_Enc_A = 1) And (temp_A = 0) And (temp_B = 1) Then
    Rotary_Stat   = CW
    Rotary_A_Lock = True
  End If
  If Rotary_A_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_A_Lock = False
    End If
  End If

  If (Rotary_Enc_B = 1) And (temp_B = 0) And (temp_A = 1) Then
    Rotary_Stat   = CCW
    Rotary_B_Lock = True
  End If
  If Rotary_B_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_B_Lock = False
    End If
  End If

  Rotary_Enc_A = temp_A
  Rotary_Enc_B = temp_B
End Function

'===============================================================================
' decode number -> 7segment-LED
'===============================================================================
Function led7seg_decode_number( num As Byte ) As Byte
  Select num
    Case 0  result = Digit_0
    Case 1  result = Digit_1
    Case 2  result = Digit_2
    Case 3  result = Digit_3
    Case 4  result = Digit_4
    Case 5  result = Digit_5
    Case 6  result = Digit_6
    Case 7  result = Digit_7
    Case 8  result = Digit_8
    Case 9  result = Digit_9
  Else
    result = Digit_N  
  EndSelect
End Function


'===============================================================================
'  number -> led_val1 to led_val6
' num * 2ms = time
'===============================================================================
Sub decode_number( num As LongWord )
  Dim tmp_val(6) As Byte
  Dim tmp_count As LongWord
  Dim i As Byte

  tmp_count = num / 5 
  tmp_val(0) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(1) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(2) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(3) = Byte(tmp_count Mod 6  ) 
  tmp_count  = tmp_count / 6
  tmp_val(4) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(5) = Byte(tmp_count Mod 6  )
  
  For i = 5 To 1 Step -1
    If tmp_val(i) = 0 Then
      tmp_val(i) = 10
    Else
      Break
    End If
  Next

  If stopwatch_mode = MODE_TIME_ATTACK_GROUP Then
    If raceing_number < 0 Then
      tmp_val(5) = -raceing_number
    Else
      tmp_val(5) = raceing_number
    End If    
  End If

  led_val1 = led7seg_decode_number( tmp_val(0) )
  led_val2 = led7seg_decode_number( tmp_val(1) )
  led_val3 = led7seg_decode_number( tmp_val(2) )
  led_val4 = led7seg_decode_number( tmp_val(3) )
  led_val5 = led7seg_decode_number( tmp_val(4) )
  led_val6 = led7seg_decode_number( tmp_val(5) )
  If led_val3 <> Digit_N Then
    led_val3 = led_val3 Or Digit_DP
  End If
  If led_val5 <> Digit_N Then
    led_val5 = led_val5 Or Digit_DP
  End If
  If stopwatch_mode = MODE_TIME_ATTACK_GROUP Then
    If raceing_number < 0 Then
      led_val6 = led_val6 Or Digit_DP
      led_val5 = Digit_N
      led_val4 = Digit_N
      led_val3 = Digit_N
      led_val2 = Digit_N
      led_val1 = Digit_E
    End If    
  End If
End Sub

'===============================================================================
'  number -> led_val1 to led_val5
' num * 2ms = time
'===============================================================================
Sub decode_number_atk( num As LongWord )
  Dim tmp_val(6) As Byte
  Dim tmp_count As LongWord
  Dim i As Byte

  tmp_count = num / 5 
  tmp_val(0) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(1) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(2) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(3) = Byte(tmp_count Mod 6  ) 
  tmp_count  = tmp_count / 6
  tmp_val(4) = Byte(tmp_count Mod 10 ) 
  
  For i = 4 To 1 Step -1
    If tmp_val(i) = 0 Then
      tmp_val(i) = 10
    Else
      Break
    End If
  Next

  led_val1 = led7seg_decode_number( tmp_val(0) )
  led_val2 = led7seg_decode_number( tmp_val(1) )
  led_val3 = led7seg_decode_number( tmp_val(2) )
  led_val4 = led7seg_decode_number( tmp_val(3) )
  led_val5 = led7seg_decode_number( tmp_val(4) )
  If led_val3 <> Digit_N Then
    led_val3 = led_val3 Or Digit_DP
  End If
  If led_val5 <> Digit_N Then
    led_val5 = led_val5 Or Digit_DP
  End If
End Sub

'===============================================================================
' decode racing number 
'===============================================================================
Sub decode_number_racenum( )
  Dim tmp_val As Byte

  If raceing_number < 0 Then
    tmp_val = -raceing_number
  Else
    tmp_val = raceing_number
  End If
  led_val6 = led7seg_decode_number( tmp_val )

  If raceing_number < 0 Then
    led_val6 = led_val6 Or Digit_DP
    led_val5 = Digit_N
    led_val4 = Digit_N
    led_val3 = Digit_N
    led_val2 = Digit_N
    led_val1 = Digit_E
  End If
End Sub

'===============================================================================
'  number -> led_val1 to led_val6
' num * 2ms = time
'===============================================================================
Sub decode_number_setmode( num As LongWord, dp_pos As Byte )
  Dim tmp_val(6)  As Byte
  Dim tmp_val2(6) As Byte
  Dim tmp_count As LongWord
  Dim i As Byte

  tmp_count = num / 5 
  tmp_val(0) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(1) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(2) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(3) = Byte(tmp_count Mod 6  ) 
  tmp_count  = tmp_count / 6
  tmp_val(4) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(5) = Byte(tmp_count Mod 6  )

  For i = 0 To 5
    tmp_val2(i) = led7seg_decode_number( tmp_val(i) )
  Next
  tmp_val2(dp_pos) = tmp_val2(dp_pos) Or Digit_DP   
  
  led_val1 = tmp_val2(0)
  led_val2 = tmp_val2(1)
  led_val3 = tmp_val2(2)
  led_val4 = tmp_val2(3)
  led_val5 = tmp_val2(4)
  led_val6 = tmp_val2(5)
End Sub

'===============================================================================
'  display start/goal Switch
'===============================================================================
Sub display_message( message_name As Byte)
  led_val1 = Digit_N
  led_val2 = Digit_N
  led_val3 = Digit_N
  led_val4 = Digit_N
  led_val5 = Digit_N
  led_val6 = Digit_N
  Select message_name
    Case MESSAGE_TIME_ATTACK_GROUP
      led_val6 = Digit_A
    Case MESSAGE_LAPTIME_SINGLE
      led_val6 = Digit_L
    Case MESSAGE_BAR
      led_val6 = Digit_MINUS
      led_val5 = Digit_MINUS
      led_val4 = Digit_MINUS
      led_val3 = Digit_MINUS
      led_val2 = Digit_MINUS
      led_val1 = Digit_MINUS
  EndSelect
End Sub

'===============================================================================
'  display count down message
'===============================================================================
Sub display_count_down_message( pos As Byte )
  led_val1 = Digit_N
  led_val2 = Digit_N
  led_val3 = Digit_N
  led_val4 = Digit_N
  led_val5 = Digit_N
  led_val6 = Digit_N
  Select pos
    Case 1 
      led_val1 = Digit_8
      led_val2 = Digit_8
      led_val3 = Digit_8
      led_val4 = Digit_8
      led_val5 = Digit_8
      led_val6 = Digit_8
    Case 2 led_val2 = Digit_8
    Case 3 led_val3 = Digit_8
    Case 4 led_val4 = Digit_8
    Case 5 led_val5 = Digit_8
    Case 6 led_val6 = Digit_8
  EndSelect
End Sub

'===============================================================================
'  input time   00:00.00
'===============================================================================
Function input_time( num As LongWord ) As LongWord
  Dim dp_pos As Byte
  Dim mul_num As LongWord

  If SW_START = SW_ON Then
    DelayMS(100)
    While( SW_START = SW_ON ) 
    Wend   
  End If
  
  dp_pos = 0
  mul_num = 1 * 5
  decode_number_setmode( num, dp_pos )
  While(true)
    '---- move decimal place ----------
    If SW2 = SW_ON Then
      DelayMS(100)
      If dp_pos = 5 Then
        dp_pos = 0
      Else
        dp_pos = dp_pos + 1
      End If
      decode_number_setmode( num, dp_pos )
      Select dp_pos
        Case 0  mul_num = 1     * 5
        Case 1  mul_num = 10    * 5
        Case 2  mul_num = 100   * 5
        Case 3  mul_num = 1000  * 5
        Case 4  mul_num = 6000  * 5
        Case 5  mul_num = 60000 * 5
      EndSelect  
      While(SW2 = SW_ON)
      Wend
      DelayMS(100)
    End If
    
    '---- rotary encoder --------------
    turn = rotary_encoder()
    Select turn
      Case CW
        If (num + mul_num) < MAX_TIME Then
          num = num + mul_num
        End If
        decode_number_setmode( num, dp_pos )
      Case CCW
        If mul_num <= num Then
          num = num - mul_num
        End If
        decode_number_setmode( num, dp_pos )
    EndSelect
    
    If SW_START = SW_ON Then
      DelayMS(100)
      Break
    End If
  Wend
  
  result = num
End Function

'===============================================================================
'  data initialize
'===============================================================================
Sub init_data()
  Dim i As Byte
  time_count = 0
  raceing_number = 0
  race_number_start  = 0
  race_number_goal   = 0
  start_time_prev = 0
  goal_time_prev  = 0
  For i = 0 To 9
    start_time(i) = 0
    goal_time(i) = 0
  Next  
End Sub

'*******************************************************************************
' Program main
'*******************************************************************************
PortB_pullups_disable  = true

ADconfig0 = 1  ' PortA Digital I/O
ADconfig1 = 1
ADconfig2 = 1 
ADconfig3 = 1

Comparator_mode2 = 1 ' compatator off
Comparator_mode1 = 1 ' compatator off
Comparator_mode0 = 1 ' compatator off
 
Input(ENCODER_A)
Input(ENCODER_B) 

Input(Ir1)
Input(Ir2) 

Input(SW_START)
Input(SW_SPLIT)

Output(RELAY)

Output(LED1)
Output(LED2)
Output(LED3)

Output(LED_DEV6)
Output(LED_DEV5)
Output(LED_DEV4)
Output(LED_DEV3)
Output(LED_DEV2)
Output(LED_DEV1)
   
Output(LED_SEG_A)
Output(LED_SEG_B)
Output(LED_SEG_C)
Output(LED_SEG_D)
Output(LED_SEG_E)
Output(LED_SEG_F)
Output(LED_SEG_G)
Output(LED_SEG_DP)

LED_DEV6 = LED_OFF
LED_DEV5 = LED_OFF
LED_DEV4 = LED_OFF
LED_DEV3 = LED_OFF
LED_DEV2 = LED_OFF
LED_DEV1 = LED_OFF

led_val1 = Digit_8
led_val2 = Digit_8
led_val3 = Digit_8
led_val4 = Digit_8
led_val5 = Digit_8
led_val6 = Digit_8
led_pos  = 1  
 
'---- TMR0(8bitMODE) initialize --------------
Timer0_prescale_bit2 = 1   ' PreScaler 1:128
Timer0_prescale_bit1 = 1   ' PreScaler 1:128
Timer0_prescale_bit0 = 0   ' PreScaler 1:128
Timer0_prescale_NOT_assign = false
Timer0_mode_8bit           = true
Timer0_interrupt_enable    = true
Timer0_use_external_clock  = false
Timer0_enable              = true
Timer0_priority            = Low_priority
 
'---- TMR2(8bit) initialize --- overflow 2ms ----
Timer2_interrupt_enable    = true 
Timer2_prescale_bit0       = 1 ' 1:16
Timer2_prescale_bit1       = 1
Timer2_postscale_bit0      = 1 ' 1:10
Timer2_postscale_bit1      = 0
Timer2_postscale_bit2      = 0
Timer2_postscale_bit3      = 1
Timer2_enable              = true
PR2 = 149
Timer2_priority            = High_priority
 
'---- interrpt -------------------------------
Enable(OnTimerHigh)
Enable(OnTimerLow)
Interrupt_priority    = true
Interrupt_HIGH_enable = true
Interrupt_LOW_enable  = true

Rotary_Enc_A = ENCODER_A
Rotary_Enc_B = ENCODER_B 
 
LED1 = LED_OFF
LED2 = LED_OFF
LED3 = LED_OFF 
RELAY = RELAY_OFF 

DelayMS(500)

init_data()

'----------- setup mode ------------------------------
stopwatch_mode = ReadByte( eepaddr_stopwatch_mode )
dead_time      = ReadLongWord( eepaddr_dead_time  )   
display_time   = ReadLongWord( eepaddr_display_time ) 

If SW_START = SW_ON Then    ' SW2 = ON
  display_message( MESSAGE_BAR )
  While( SW_START = SW_ON ) 
  Wend
  DelayMS(2000)

  '------- TIME ATTACK / LAPTIME ------------------
  If stopwatch_mode = MODE_LAPTIME_SINGLE Then
    display_message( MESSAGE_LAPTIME_SINGLE )
  Else
    display_message( MESSAGE_TIME_ATTACK_GROUP )
  EndIf  
  While(true) 
    turn = rotary_encoder()
    Select turn
      Case CW 
        If stopwatch_mode = MODE_TIME_ATTACK_GROUP Then
          stopwatch_mode = MODE_LAPTIME_SINGLE
        End If
        display_message( MESSAGE_LAPTIME_SINGLE )
      Case CCW
        If stopwatch_mode = MODE_LAPTIME_SINGLE Then
          stopwatch_mode = MODE_TIME_ATTACK_GROUP
        End If
        display_message( MESSAGE_TIME_ATTACK_GROUP )
    EndSelect
    If SW_START = SW_ON Then
      DelayMS(100)
      While( SW_START = SW_ON ) 
      Wend           
      Break
    End If 
  Wend
  WriteByte(eepaddr_stopwatch_mode, stopwatch_mode)
 
  '------- input dead time    00:00.00 --------------
  dead_time = input_time(dead_time)
  WriteLongWord(eepaddr_dead_time, dead_time)

  '------- input display time 00:00.00 --------------
  LED1 = LED_ON
  display_time = input_time(display_time)
  LED1 = LED_OFF
  WriteLongWord(eepaddr_display_time, display_time)
   
  DelayMS(100)
  While( SW_START = SW_ON ) 
  Wend           
End If
DelayMS(500)
'----------- setup mode END --------------------------

If stopwatch_mode = MODE_TIME_ATTACK_GROUP Then 
  decode_number_racenum()
  decode_number_atk(0)
Else
  decode_number(0)
End If

display_down_counter = 0
While( true )
  '---- START / STOP switch ---------------
  ' not use

  '---- TIME ATTACK MODE ---------------------------------------------------
  If stopwatch_mode = MODE_TIME_ATTACK_GROUP Then  
    '---- Ir SENSOR1 -------------------------
    If Ir1 = SW_ON Then
      temp_start_time = time_count
      If (temp_start_time - start_time_prev) > dead_time Then  ' dead time test
        start_time(race_number_start) = temp_start_time
        raceing_number = raceing_number + 1
        decode_number_racenum()
        If race_number_start < 9 Then
          race_number_start = race_number_start + 1
        Else
          race_number_start = 0
        End If
      End If    
      start_time_prev = temp_start_time
    End If
    
    '---- Ir SENSOR2 -------------------------
    If Ir2 = SW_ON Then
      temp_goal_time = time_count
      If (temp_goal_time - goal_time_prev) > dead_time Then  ' dead time test  
        goal_time(race_number_goal) = temp_goal_time
        display_down_counter = display_time
        raceing_number = raceing_number - 1
        decode_number_atk( goal_time(race_number_goal) - start_time(race_number_goal) )
        decode_number_racenum()
        If race_number_goal < 9 Then
          race_number_goal = race_number_goal + 1
        Else 
          race_number_goal = 0
        End If        
      End If
      goal_time_prev = temp_goal_time
    End If
    
    '---- display time ----------------------  
    If (raceing_number >= 1) And (display_down_counter = 0) Then
      decode_number_atk( time_count - start_time(race_number_goal) )
    End If
  End If

  If stopwatch_mode = MODE_LAPTIME_SINGLE Then  
    '---- Ir SENSOR1 -------------------------
    If Ir1 = SW_ON Then
      temp_start_time = time_count
      If (temp_start_time - start_time_prev) > dead_time Then  ' dead time test
        For i = 9  To 1 Step -1
          start_time(i) = start_time(i-1)  
        Next
        start_time(0) = temp_start_time
        If raceing_number = 1 Then
          display_down_counter = display_time
          decode_number( start_time(0) - start_time(1) )
        End If
      End If
      raceing_number = 1
      start_time_prev = temp_start_time
    End If
    '---- display time ----------------------  
    If (raceing_number = 1) And (display_down_counter = 0) Then
      decode_number( time_count - start_time(0) )
    End If
  End If
                
    
  '---- SPLIT / RESET switch ---------------
  If SW_SPLIT = SW_ON Then
    time_count = 0
    init_data()
    decode_number( 0 )
  End If  
Wend

'--------- E N D  O F  P R O G R A M ------------------------------------------- 
 
